@@ -283,6 +283,12 @@ class isInstanceOf(Validator):
     def test(self,x):
         return isinstance(x,self._klass)
 
+class isSubclassOf(Validator):
+    def __init__(self,klass=None):
+        self._klass = klass
+    def test(self,x):
+        return issubclass(x,self._klass)
+
 class matchesPattern(Validator):
     """Matches value, or its string representation, against regex"""
     def __init__(self, pattern):
@@ -340,6 +346,8 @@ isNoneOrInt = NoneOr(isInt,'isNoneOrInt')
 isNumberOrNone = _isNumberOrNone()
 isTextAnchor = OneOf('start','middle','end','boxauto')
 isListOfNumbers = SequenceOf(isNumber,'isListOfNumbers')
+isListOfNoneOrNumber = SequenceOf(isNumberOrNone,'isListOfNoneOrNumber')
+isListOfListOfNoneOrNumber = SequenceOf(isListOfNoneOrNumber,'isListOfListOfNoneOrNumber')
 isListOfNumbersOrNone = _isListOfNumbersOrNone()
 isListOfShapes = _isListOfShapes()
 isListOfStrings = SequenceOf(isString,'isListOfStrings')
